﻿// Ribade ülesanne, N * logN lahendus, ülalt alla, lõikude puuga

using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace Ribad2
{
    public class Ribad2
    {
        private static List<int> _pointList = new List<int>();
        private static List<Ribbon> _result;

        private class Ribbon
        {
            public Ribbon(int start, int end, int color)
            {
                Start = start;
                End = end;
                Color = color;
            }

            public readonly int Start;
            public int End;
            public int Color;
        }

        // tree of line segments (not points)
        private class TreeNode
        {
            public int Color;

            public int LeftPoint;
            public int RightPoint;

            public int LeftEnd; // extent of left subtree
            public int RightEnd; // extent of right subtree

            public bool CoversLeft;
            public bool CoversRight;

            public TreeNode LeftNode;
            public TreeNode RightNode;
        }

        private static void Main()
        {
            HashSet<int> pointSet = new HashSet<int>();
            List<Ribbon> ribbons = new List<Ribbon>();

            using (StreamReader reader = new StreamReader("ribasis.txt"))
            {
                int[] parts = reader.ReadLine().Split().Select(int.Parse).ToArray();
                int l = parts[0];
                int n = parts[1];

                pointSet.Add(0);
                pointSet.Add(l);

                for (int i = 0; i < n; i++)
                {
                    parts = reader.ReadLine().Split().Select(int.Parse).ToArray();
                    Ribbon newRibbon = new Ribbon(parts[1], parts[2], parts[0]);
                    ribbons.Add(newRibbon);
                    pointSet.Add(newRibbon.Start);
                    pointSet.Add(newRibbon.End);
                }
            }

            _pointList = pointSet.ToList();
            _pointList.Sort();

            TreeNode root = MakeTree(0, _pointList.Count - 1);

            ribbons.Reverse();
            foreach (Ribbon ribbon in ribbons)
            {
                SetRibbon(root, ribbon);
            }

            _result = new List<Ribbon>();
            WriteTree(root);

            using (StreamWriter writer = new StreamWriter("ribaval.txt"))
            {
                foreach (Ribbon ribbon in _result)
                {
                    writer.WriteLine("{0} {1}", ribbon.Color, ribbon.End - ribbon.Start);
                }
            }
        }

        private static void WriteTree(TreeNode node)
        {
            if (node.LeftNode != null && !node.CoversLeft)
            {
                WriteTree(node.LeftNode);
            }

            Ribbon ribbon = new Ribbon(node.CoversLeft ? node.LeftEnd : node.LeftPoint,
                                       node.CoversRight ? node.RightEnd : node.RightPoint, node.Color);
            if (ribbon.Color == -1)
            {
                ribbon.Color = 0;
            }

            if (_result.Count > 0 && _result[_result.Count - 1].Color == ribbon.Color)
            {
                _result[_result.Count - 1].End = ribbon.End;
            }
            else
            {
                _result.Add(ribbon);
            }

            if (node.RightNode != null && !node.CoversRight)
            {
                WriteTree(node.RightNode);
            }
        }

        private static void SetRibbon(TreeNode node, Ribbon ribbon)
        {
            // 0 == täiesti värvimata haru, -1 = osaliselt värvitud, värvimata juurtipuga haru
            if (node.Color <= 0)
            {
                if (node.Color == 0 && ribbon.Start <= node.LeftEnd && ribbon.End >= node.RightPoint)
                {
                    node.CoversLeft = true;
                }
                if (node.Color == 0 && ribbon.Start <= node.LeftPoint && ribbon.End >= node.RightEnd)
                {
                    node.CoversRight = true;
                }
                if (ribbon.Start <= node.LeftPoint && ribbon.End >= node.RightPoint)
                {
                    node.Color = ribbon.Color;
                }
                else
                {
                    node.Color = -1;
                }
            }

            if (ribbon.Start < node.LeftPoint && !node.CoversLeft && node.LeftNode != null)
            {
                SetRibbon(node.LeftNode, ribbon);
            }
            if (ribbon.End > node.RightPoint && !node.CoversRight && node.RightNode != null)
            {
                SetRibbon(node.RightNode, ribbon);
            }
        }

        private static TreeNode MakeTree(int startIx, int endIx)
        {
            int medIx = (startIx + endIx) / 2;
            TreeNode node = new TreeNode
            {
                LeftEnd = _pointList[startIx],
                RightEnd = _pointList[endIx],
                LeftPoint = _pointList[medIx],
                RightPoint = _pointList[medIx + 1]
            };
            if (startIx < medIx)
            {
                node.LeftNode = MakeTree(startIx, medIx);
            }
            if (endIx > medIx + 1)
            {
                node.RightNode = MakeTree(medIx + 1, endIx);
            }

            return node;
        }
    }
}
